var util = require('util')
  , Store = require('jsr-store')
  , Database = Store.Database
  , StoreError = require('jsr-error')
  , DatabaseIndex = StoreError.DatabaseIndex
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the SELECT command.
 */
function Select() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Select, AbstractCommand);

/**
 *  Respond to the SELECT command.
 */
function execute(req, res) {

  // if we have a sparse db store we need
  // to ensure a database exists
  if(!this.store.databases[req.args[0]]) {
    this.store.getDatabase(req.args[0], {pattern: this.pattern});
  }

  // update the connection with their selected db index
  req.conn.db = req.args[0];

  res.send(null, Constants.OK);
}

/**
 *  Validate the SELECT command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var index = parseInt(args[0]);
  if(isNaN(index) || index < 0 || index >= info.conf.databases) {
    throw DatabaseIndex;
  }
  // update the index with the integer value
  // connection expects an integer
  args[0] = index;
}

Select.prototype.validate = validate;
Select.prototype.execute = execute;

module.exports = new Select(Constants.MAP.select);
