var util = require('util')
  , Constants = require('jsr-constants')
  , getEncoding = require('jsr-persistence').encoding
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the OBJECT command.
 */
function KeyObject() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(KeyObject, AbstractCommand);

/**
 *  Respond to the REFCOUNT subcommand.
 */
function refcount(req, res) {
  var obj = req.db.getKey(req.args[0], req);
  if(obj === undefined) return res.send(null, null);
  res.send(null, -1);
}

/**
 *  Respond to the IDLETIME subcommand.
 */
function idletime(req, res) {
  var obj = req.db.getRawKey(req.args[0], req)
    , diff;
  if(obj === undefined) return res.send(null, null);
  diff = Date.now() - obj.t;
  diff = Math.round(diff / 1000);
  res.send(null, diff);
}

/**
 *  Respond to the ENCODING subcommand.
 */
function encoding(req, res) {
  var obj = req.db.getRawKey(req.args[0], req);
  if(obj === undefined) return res.send(null, null);
  res.send(null, getEncoding(
    req.db.getType(req.args[0]), obj.v, this.state.conf));
}

/**
 *  Respond to the OBJECT command.
 */
function execute(req, res) {
  this.delegate(req, res);
}

KeyObject.prototype.execute = execute;

// subcommands
KeyObject.prototype.refcount = refcount;
KeyObject.prototype.idletime = idletime;
KeyObject.prototype.encoding = encoding;

module.exports = new KeyObject(Constants.MAP.object);
