var util = require('util')
  , StoreError = require('jsr-error')
  , CommandArgLength = StoreError.CommandArgLength
  , CommandSyntax = StoreError.CommandSyntax
  , PatternCompile = StoreError.PatternCompile
  , IntegerRange = StoreError.IntegerRange
  , Scanner = require('../../../scanner')
  , Constants = require('jsr-constants')
  , Pattern = require('jsr-pattern')
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the SCAN command.
 */
function Scan() {
  AbstractCommand.apply(this, arguments);

  // offset used to check MATCH and COUNT
  // hscan, zscan and sscan will increment this
  // as they accept a key argument
  this.offset = 1;
}

util.inherits(Scan, AbstractCommand);

/**
 *  Respond to the SCAN command.
 */
function execute(req, res) {
  var scanner = new Scanner(
    req.db.getKeys(), req.info.cursor, req.info.pattern, req.info.count);
  scanner.scan(req, res);
}

/**
 *  Validate the SCAN command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var offset = info.exec.offset
    , arg = args[offset];

  if(args.length > offset + 4) {
    throw new CommandArgLength(info.exec.definition.name);
  }

  // cursor already parsed to integer by numeric validation
  info.cursor = args[offset - 1]

  //console.dir(args);

  if(arg !== undefined) {
    arg = (('' + arg).toLowerCase());
    while(arg !== undefined) {
      //console.dir(arg);
      if(arg !== Constants.SCAN.MATCH && arg !== Constants.SCAN.COUNT) {
        throw CommandSyntax;
      }else if(arg === Constants.SCAN.MATCH) {
        /* istanbul ignore next: really tough to get a pattern compile error! */
        try {
          info.pattern = Pattern.compile(args[offset + 1]);
        }catch(e) {
          throw PatternCompile;
        }
      // should be on count arg
      }else{
        info.count = parseInt('' + args[offset + 1]);
        if(isNaN(info.count)) {
          throw IntegerRange;
        }
      }
      offset+=2;
      arg = args[offset] ? '' + args[offset] : undefined;
    }
  }
}

Scan.prototype.validate = validate;
Scan.prototype.execute = execute;

Scan.definition = Constants.MAP.scan;

module.exports = Scan;
