var util = require('util')
  , Constants = require('jsr-constants')
  , StoreError = require('jsr-error')
  , NoSuchKey = StoreError.NoSuchKey
  , IndexRange = StoreError.IndexRange
  , AbstractCommand = require('jsr-exec').DatabaseCommand;

/**
 *  Handler for the LSET command.
 */
function ListSet() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(ListSet, AbstractCommand);

/**
 *  Validate the LSET command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var index = parseInt(args[1])
    , value = info.db.getKey(args[0], info);
  if(value === undefined) {
    throw NoSuchKey;
  }else if(isNaN(index) || (index < 0 || index >= value.getLength())) {
    throw IndexRange;
  }
  args[1] = index;
}

ListSet.prototype.validate = validate;

module.exports = new ListSet(Constants.MAP.lset);
