var util = require('util')
  , StoreError = require('jsr-error')
  , IntegerRange = StoreError.IntegerRange
  , InvalidBit = StoreError.InvalidBit
  , CommandArgLength = StoreError.CommandArgLength
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').DatabaseCommand;

/**
 *  Handler for the BITPOS command.
 */
function BitPosition() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(BitPosition, AbstractCommand);

/**
 *  Validate the BITPOS command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var bit = parseInt('' + args[1])
    , start
    , end;

  if(args.length > 4) {
    throw new CommandArgLength(cmd);
  }

  if(bit !== 0 && bit !== 1) throw InvalidBit;
  if(args.length > 2) {
    start = parseInt('' + args[2]);
    if(isNaN(start)) throw IntegerRange;
    args[2] = start;
    if(args.length > 3) {
      end = parseInt('' + args[3]);
      if(isNaN(end)) throw IntegerRange;
      args[3] = end;
    }
  }

  args[1] = bit;
}

BitPosition.prototype.validate = validate;

module.exports = new BitPosition(Constants.MAP.bitpos);
