var util = require('util')
  , StoreError = require('jsr-error')
  , CommandArgLength = StoreError.CommandArgLength
  , CommandSyntax = StoreError.CommandSyntax
  , IntegerRange = StoreError.IntegerRange
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').DatabaseCommand;

/**
 *  Handler for the SET command.
 */
function Set() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Set, AbstractCommand);

/**
 *  Validate the SET command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var offset = 2
    , arg
    , expires;

  // need string args (except for key, value)
  args.forEach(function(a, i, arr) {
    if(i > 1) {
      arr[i] = '' + a;
    }
  })

  if(args.length > 7) {
    throw new CommandArgLength(cmd);
  }else if(args.length > 2) {
    arg = args[offset];
    while(arg !== undefined) {
      arg = arg.toLowerCase();
      args[offset] = arg;
      if(arg !== Constants.EX
        && arg !== Constants.PX
        && arg !== Constants.NX
        && arg !== Constants.XX) {
        throw CommandSyntax;
      }
      if(arg === Constants.EX || arg === Constants.PX) {
        expires = parseInt('' + args[offset + 1]);
        if(isNaN(expires)) {
          throw IntegerRange;
        }
        args[++offset] = expires;
      }
      arg = args[++offset];
    }
  }
}

Set.prototype.validate = validate;

module.exports = new Set(Constants.MAP.set);
