var util = require('util')
  , StoreError = require('jsr-error')
  , CommandArgLength = StoreError.CommandArgLength
  , InvalidFloat = StoreError.InvalidFloat
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').DatabaseCommand;

/**
 *  Handler for the ZADD command.
 */
function SortedSetAdd() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(SortedSetAdd, AbstractCommand);

/**
 *  Validate the ZADD command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var i
    , num;
  if((args.length - 1) % 2 !== 0) {
    throw new CommandArgLength(cmd);
  }

  for(i = 1;i < args.length;i += 2) {
    num = parseFloat('' + args[i]);
    if(isNaN(num)) {
      throw InvalidFloat;
    }
    args[i] = num;
  }
}

SortedSetAdd.prototype.validate = validate;

module.exports = new SortedSetAdd(Constants.MAP.zadd);
