var util = require('util')
  , Constants = require('jsr-constants')
  , StoreError = require('jsr-error')
  , UnknownSubcommand = StoreError.UnknownSubcommand
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the PUBSUB command.
 */
function PubSub() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(PubSub, AbstractCommand);

/**
 *  Respond to the CHANNELS subcommand.
 */
function channels(req, res) {
  res.send(null, this.state.pubsub.getChannels(req.args[0]));
}

/**
 *  Respond to the NUMSUB subcommand.
 */
function numsub(req, res) {
  res.send(null, this.state.pubsub.getChannelList(req.args));
}

/**
 *  Respond to the NUMPAT subcommand.
 */
function numpat(req, res) {
  res.send(null, this.state.pubsub.getPatternSubscriberCount());
}

/**
 *  Validate the PUBSUB subcommands.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var sub = info.command.sub
    , cmd = sub.cmd
    , args = sub.args
  if(('' + cmd) === Constants.SUBCOMMAND.pubsub.channels.name) {
    if(args.length > 1) {
      throw UnknownSubcommand;
    }
  }
}

/**
 *  Respond to the PUBSUB command.
 */
function execute(req, res) {
  this.delegate(req, res);
}

PubSub.prototype.validate = validate;
PubSub.prototype.execute = execute;

// subcommands
PubSub.prototype.channels = channels;
PubSub.prototype.numsub = numsub;
PubSub.prototype.numpat = numpat;

module.exports = new PubSub(Constants.MAP.pubsub);
