var util = require('util')
  , StoreError = require('jsr-error')
  , SaveInProgress = StoreError.SaveInProgress
  , Constants = require('jsr-constants')
  , Persistence = require('jsr-persistence')()
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the BGSAVE command.
 */
function BackgroundSave() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(BackgroundSave, AbstractCommand);

/**
 *  Respond to the BGSAVE command.
 */
function execute(req, res) {
  this.log.notice('background saving started by pid %s', process.pid);
  function onSave(err) {
    /* istanbul ignore next: not prepared to mock this right now */
    if(err) return this.log.warning('background save failed: %s', err.message);
    this.log.notice('background saving terminated with success')
  }
  Persistence.save(this.state.store, this.state.conf, onSave.bind(this));
  res.send(null, Constants.BGSAVE);
}

/**
 *  Validate the BGSAVE command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  if(Persistence.saving) throw SaveInProgress;
}

BackgroundSave.prototype.validate = validate;
BackgroundSave.prototype.execute = execute;

module.exports = new BackgroundSave(Constants.MAP.bgsave);
