var util = require('util')
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').Command
  , COMMAND_LIST = null;

/**
 *  Handler for the COMMAND command.
 */
function Command() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Command, AbstractCommand);

/**
 *  Helper to lazily build the command list.
 */
function getList() {
  if(!COMMAND_LIST) {
    COMMAND_LIST = [];
    for(var z in this.execs) {
      COMMAND_LIST.push(this.execs[z].definition.getDefinition());
    }
  }
  return COMMAND_LIST;
}

/**
 *  Respond to the INFO subcommand.
 */
function info(req, res) {
  var list = []
    , i
    , exec;
  for(i = 0;i < req.args.length;i++) {
    exec = this.execs[req.args[i]];
    list.push(exec ? exec.definition.getDefinition() : null);
  }
  res.send(null, list);
}

/**
 *  Respond to the GETKEYS subcommand.
 *
 *  Current version redis@2.8.19 does not actually appear to
 *  implement this: `command getkeys get key` results in an error.
 *
 *  Therefore this implementation is a best guess as to the intended
 *  behaviour.
 *
 *  This command cannot fail with an error so we just return an empty list
 *  for target commands that do not accept keys.
 *
 *  Examples:
 *
 *  command getkeys set key val         // ['key']
 *  command getkeys mset k1 v1 k2 v2    // ['k1', 'k2']
 *  command getkeys hset hash field val // ['hash']
 */
function getkeys(req, res) {
  var keys = []
    // target command
    , tcmd = req.args[0]
    , def = Constants.MAP[tcmd];
  if(def) {
    keys = def.getKeys(req.args.slice(1));
  }
  res.send(null, keys);
}

/**
 *  Respond to the COUNT subcommand.
 */
function count(req, res) {
  var list = getList.call(this);
  return res.send(null, list.length);
}

/**
 *  Respond to the COMMAND command.
 */
function execute(req, res) {
  if(req.info.command.sub) {
    this.delegate(req, res);
  // no subcommand return command list
  }else{
    var list = getList.call(this);
    return res.send(null, list);
  }
}

Command.prototype.execute = execute;

// subcommands
Command.prototype.info = info;
Command.prototype.count = count;
Command.prototype.getkeys = getkeys;

module.exports = new Command(Constants.MAP.command);
