var util = require('util')
  , Constants = require('jsr-constants')
  , Persistence = require('jsr-persistence')()
  , StoreError = require('jsr-error')
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the DEBUG command.
 */
function Debug() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Debug, AbstractCommand);

/**
 *  Respond to the ERROR subcommand.
 */
function error(req, res) {
  res.send(new Error('' + req.args[0]));
}

/**
 *  Respond to the DIFF subcommand.
 *
 *  Non-standard debug subcommand that returns a diff between the list
 *  of standard redis commands and the commands loaded.
 *
 *  Primarily useful to determine which commands have not yet been
 *  implemented, but in a scenario where commands have been renamed, deleted
 *  or custom commands loaded this could also be useful.
 */
function diff(req, res) {
  // defined and loaded commands
  var cmds = this.execs
    // map of all standard top-level commands
    , map = Constants.MAP
    // output multi bulk reply list
    , list = []
    , k;

  for(k in map) {
    if(!cmds[k]) list.push(k);
  }

  res.send(null, list);
}

/**
 *  Respond to the RELOAD subcommand.
 */
function reload(req, res) {
  var log = this.log;
  if(Persistence.loading) {
    return res.send(new Error('database load in progress'));
  }
  Persistence.load(this.state.store, this.state.conf,
    function onLoad(err, time, diff, version) {
      if(err) return log.warning('db reload error: %s', err.message);
      log.notice(
        'db reloaded by DEBUG RELOAD: %s seconds (v%s)', time, version);
    }
  );
  res.send(null, Constants.OK);
}

/**
 *  Validate the DEBUG subcommands.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
}

/**
 *  Respond to the DEBUG command.
 */
function execute(req, res) {
  this.delegate(req, res);
}

Debug.prototype.validate = validate;
Debug.prototype.execute = execute;

// subcommands
Debug.prototype.error = error;
Debug.prototype.diff = diff;
Debug.prototype.reload = reload;

module.exports = new Debug(Constants.MAP.debug);
