var util = require('util')
  , Constants = require('jsr-constants')
  , StoreError = require('jsr-error')
  , CommandSyntax = StoreError.CommandSyntax
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the SHUTDOWN command.
 */
function Shutdown() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Shutdown, AbstractCommand);

/**
 *  Respond to the SHUTDOWN command.
 */
function execute(req, res) {
  var save = undefined;
  if(req.args[0] !== undefined) {
    save = req.args[0] === true;
  }
  this.shutdown({save: save, code: 0}, process.exit);
}

/**
 *  Validate the SHUTDOWN command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  if(args[0]) {
    var arg = ('' + args[0]).toLowerCase();
    if(arg !== Constants.SHUTDOWN.SAVE && arg !== Constants.SHUTDOWN.NOSAVE) {
      throw CommandSyntax;
    }

    // rewrite with boolean
    args[0] = (arg === Constants.SHUTDOWN.SAVE);
  }
}

Shutdown.prototype.execute = execute;
Shutdown.prototype.validate = validate;

module.exports = new Shutdown(Constants.MAP.shutdown);
