var util = require('util')
  , StoreError = require('jsr-error')
  , CommandArgLength = StoreError.CommandArgLength
  , IntegerRange = StoreError.IntegerRange
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the SLOWLOG command.
 */
function SlowLog() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(SlowLog, AbstractCommand);

/**
 *  Respond to the GET subcommand.
 */
function get(req, res) {
  res.send(null, this.state.slowlog.get(req.args[0]));
}

/**
 *  Respond to the LEN subcommand.
 */
function len(req, res) {
  res.send(null, this.state.slowlog.len());
}

/**
 *  Respond to the RESET subcommand.
 */
function reset(req, res) {
  this.state.slowlog.reset();
  res.send(null, Constants.OK);
}

/**
 *  Validate the SLOWLOG subcommands.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  var sub = info.command.sub
    , scmd = ('' + sub.cmd)
    , args = sub.args
    , amount;

  if(scmd === Constants.SUBCOMMAND.slowlog.get.name && args.length) {
    if(args.length > 1) {
      throw new CommandArgLength(cmd, scmd);
    }
    amount = parseInt('' + args[0]);
    if(isNaN(amount)) {
      throw IntegerRange;
    }
    args[0] = amount;
  }
}

/**
 *  Respond to the SLOWLOG command.
 */
function execute(req, res) {
  this.delegate(req, res);
}

SlowLog.prototype.validate = validate;
SlowLog.prototype.execute = execute;

// subcommands
SlowLog.prototype.get = get;
SlowLog.prototype.len = len;
SlowLog.prototype.reset = reset;

module.exports = new SlowLog(Constants.MAP.slowlog);
