var util = require('util')
  , StoreError = require('jsr-error')
  , DiscardWithoutMulti = StoreError.DiscardWithoutMulti
  , Constants = require('jsr-constants')
  , AbstractCommand = require('jsr-exec').Command;

/**
 *  Handler for the DISCARD command.
 */
function Discard() {
  AbstractCommand.apply(this, arguments);
}

util.inherits(Discard, AbstractCommand);

/**
 *  Respond to the DISCARD command.
 */
function execute(req, res) {
  // unwatch keys, clear references,
  // and destroy the transaction reference on the conn
  // and send the reply
  req.conn.transaction.destroy(req, res, null, Constants.OK);
}

/**
 *  Validate the DISCARD command.
 */
function validate(cmd, args, info) {
  AbstractCommand.prototype.validate.apply(this, arguments);
  if(!info.conn.transaction) {
    throw DiscardWithoutMulti;
  }
}

Discard.prototype.validate = validate;
Discard.prototype.execute = execute;

module.exports = new Discard(Constants.MAP.discard);
