var spawn = require('child_process').spawn
  , path = require('path')
  , EOL = require('os').EOL
  , Constants = require('jsr-constants')
  , log = require('jsr-log')()
  , cmd = path.basename(process.execPath)
  , fs = require('fs')
  , ConfigKey = require('jsr-conf').Types
  , flag = '--daemon'

/**
 *  Create the daemon process.
 */
function daemonize(file, conf, args) {
  /* istanbul ignore next: never parse argv in test env */
  args = args || process.argv.slice(2);
  args.unshift(file);
  var logfile = conf.get(ConfigKey.LOGFILE) || '/dev/null'
  //var logfile = conf.get(ConfigKey.LOGFILE) || 'daemon.log'
    , out = fs.openSync(logfile, 'a')
    , err = fs.openSync(logfile, 'a')
    , opts = {
      detached: true,
      stdio: ['ignore', out, err],
      cwd: process.cwd(),
      env: process.env
    };

  //console.dir(logfile);
  //console.dir(args);

  var ps = spawn(cmd, args, opts);
  /* istanbul ignore next: always in test env */
  if(process.env.NODE_ENV !== Constants.TEST) {
    process.exit(0);
  }
  return ps.pid;
}

/**
 *  Write the pid file.
 */
function write(conf) {
  var contents = process.pid + EOL
    , file = conf.get(ConfigKey.PIDFILE);
  // can be the empty string
  if(file) {
    try {
      fs.writeFileSync(file, contents);
      log.notice('wrote pid %s to %s', process.pid, file);
    }catch(e) {
      log.warning('failed to write pid file (%s): %s', file, e.message);
    }
  }
}

/**
 *  Remove the pid file.
 */
function remove(conf) {
  var file = conf.get(ConfigKey.PIDFILE);
  // can be the empty string
  if(file) {
    log.notice('removing pid file %s', file);
    try {
      fs.unlinkSync(file);
    }catch(e) {
      log.warning('failed to remove pid file (%s): %s', file, e.message);
    }
  }
}

var pid = {
  write: write,
  remove: remove
}

daemonize.FLAG = flag;
daemonize.pid = pid;

module.exports = daemonize;
