/**
 *  Encapsulate a connection request.
 *
 *  @param conn The connection handling the request.
 *  @param cmd The command to execute.
 *  @param args The command arguments.
 */
function Request(conn, cmd, args) {

  // current connection
  this.conn = conn;

  // command name
  this.cmd = cmd;

  // command arguments may be modified
  // during validation
  this.args = args;

  // raw command arguments, injected by the server
  this.raw = null;

  // server will inject this
  // based upon the selected db index
  // for the connection
  this.db = null;

  // info object decorated with additional
  // information during validation, server will
  // inject this
  this.info = null;

  // server configuration object
  this.conf = null;

  // alias to the command definition
  this.def = null;

  // mark this request as coming from a script
  this.script = false;
}

/**
 *  Destroy this request.
 */
function destroy() {
  this.conn = null;
  this.cmd = null;
  this.args = null;
  this.db = null;
  this.info = null;
  this.conf = null;
  this.def = null;
}

Request.prototype.destroy = destroy;

module.exports = Request;
