var Constants = require('jsr-constants')
  , QUEUED = Constants.QUEUED;

/**
 *  Encapsulate a connection response.
 *
 *  When a callback function is passed the outbound
 *  response is passed to the callback and nothing is
 *  returned to the client.
 *
 *  Used during transaction execution to build an array of replies.
 *
 *  @param conn The connection.
 *  @param cb A callback function.
 */
function Response(conn, cb) {
  this.conn = conn;
  this.cb = cb;
  this.before = null;
}

/**
 *  Send a response to the client that made the request.
 *
 *  @param err An error instance.
 *  @param reply A reply array, string, integer or null.
 *  @param cb A write callback.
 */
function send(err, reply, cb) {

  // invoke before function, used to end a slowlog
  // timer before sending output
  if(this.before) this.before();

  if(this.cb) {
    return this.cb(err, reply);
  }
  this.conn.write(err || reply, cb);
}

/**
 *  Destroy this response.
 *
 *  Further calls to send will result in an exception.
 */
function destroy() {
  this.conn = null;
  this.cb = null;
}

Response.prototype.send = send;
Response.prototype.destroy = destroy;

module.exports = Response;
