/**
 *  Encapsulates the scan logic.
 *
 *  @param target The target array of keys.
 *  @param cursor The user-supplied cursor position.
 *  @param pattern A regular expression pattern if match was specified.
 *  @param count A user-specified count option.
 *  @param map An object used to lookup values (hscan and zscan).
 *  @param stringify Convert map values to strings (zscan).
 */
function Scanner(target, cursor, pattern, count, map, stringify) {
  this.target = target;
  this.cursor = cursor;
  this.pattern = pattern;
  this.count = count || 10;
  this.map = map;
  this.stringify = stringify;

  // clamp the count
  this.count = Math.max(this.count, 10);
  this.count = Math.min(this.count, 1000);
}

/**
 *  Perform the scan and send the response to the client.
 */
function scan(req, res) {
  var records = []
    , position
    , i
    , key
    , inc
    , map = this.map
    , len = this.cursor + this.count;

  for(i = this.cursor; i < len;i++) {
    if(i === this.target.length) {
      position = 0;
      break;
    }
    key = '' + this.target[i];
    inc = !this.pattern || (this.pattern && this.pattern.test(key));
    if(inc) {
      records.push(key);
      if(map) records.push(this.stringify ? '' + map[key] : map[key]);
    }
    position = i + 1;
  }

  res.send(null, [position, records]);
}

Scanner.prototype.scan = scan;

module.exports = Scanner;
