var proc = require('getrusage');

/**
 *  Executes periodically based on the hz configuration field.
 *
 *  Dispatches a tick event via the process.
 */
function Scheduler() {
  this._intervals = {};
}

/**
 *  Start the scheduler running.
 *
 *  @param hz The interval in seconds.
 */
function start(hz) {
  hz = Math.abs(Math.round(hz));
  var ms = 1000 / hz;

  this.stop();

  function dispatch() {
    // where are we in the current second
    var pos = new Date().getMilliseconds()
      , tick = Math.floor(pos / ms);
    process.emit('tick', tick);
  }

  this._intervals.tick = setInterval(dispatch, ms);
}

/**
 *  Stop the scheduler from emitting the tick event.
 */
function stop() {
  clearInterval(this._intervals.tick);
}

/**
 *  Start the debug log interval.
 */
function startDebugLog() {
  var ms = 5000;
  this.stopDebugLog();
  function dispatch() {
    process.emit('debug', proc.usage(), process.memoryUsage());
  }
  this._intervals.debug = setInterval(dispatch, ms);
}

/**
 *  Stop the debug log interval.
 */
function stopDebugLog() {
  clearInterval(this._intervals.debug);
}

/**
 *  Stop all intervals and remove all listeners.
 */
function kill() {
  for(var k in this._intervals) {
    process.removeAllListeners(k);
    clearInterval(this._intervals[k]);
    delete this._intervals[k];
  }
}

Scheduler.prototype.kill = kill;

Scheduler.prototype.start = start;
Scheduler.prototype.stop = stop;

Scheduler.prototype.startDebugLog = startDebugLog;
Scheduler.prototype.stopDebugLog = stopDebugLog;

module.exports = Scheduler;
