/**
 *  Encapsulates the server statistics.
 */
function Stats(state) {
  this.state = state;
  this.clear();
  this.onConnect = onConnect.bind(this);
  this.onDisconnect = onDisconnect.bind(this);
  this.onRejected = onRejected.bind(this);
  this.onHit = onHit.bind(this);
  this.onMiss = onMiss.bind(this);
  this.onExpired = onExpired.bind(this);
  this.onExpired = onExpired.bind(this);
  this.addStateListeners();
}

/**
 *  Listener for the connect event.
 */
function onConnect() {
  this.connected++;
  this.connections++;
}

/**
 *  Listener for the disconnect event.
 */
function onDisconnect() {
  this.connected--;
}

/**
 *  Listener for the rejected connection event.
 */
function onRejected(key) {
  this.rejected++;
}

/**
 *  Listener for the key hit event.
 */
function onHit(key) {
  this.hits++;
}

/**
 *  Listener for the key miss event.
 */
function onMiss(key) {
  this.misses++;
}

/**
 *  Listener for the key expired event.
 */
function onExpired(key) {
  this.expired++;
}

/**
 *  @private
 *
 *  Listen for events to keep track of statstics.
 */
function addStateListeners() {
  this.state.on('connect', this.onConnect);
  this.state.on('disconnect', this.onDisconnect);
  this.state.on('rejected', this.onRejected);
  this.state.store.on('hit', this.onHit);
  this.state.store.on('miss', this.onMiss);
  this.state.store.on('expired', this.onExpired);
}

/**
 *  @private
 *
 *  Stop listening for statistics changes.
 */
function removeStateListeners() {
  this.state.removeListener('connect', this.onConnect);
  this.state.removeListener('disconnect', this.onDisconnect);
  this.state.removeListener('rejected', this.onRejected);
  this.state.store.removeListener('hit', this.onHit);
  this.state.store.removeListener('miss', this.onMiss);
  this.state.store.removeListener('expired', this.onExpired);
}

/**
 *  Clear all statistics.
 */
function clear() {
  this.reset();

  // current number of connections
  this.connected = 0;

  // number of global pubsub channels
  this.pubsub_channels = 0;

  // number of global pubsub patterns
  this.pubsub_patterns = 0;

  // bytes in and out
  this.ibytes = 0;
  this.obytes = 0;
}

/**
 *  Reset the server statistics.
 *
 *  Suitable for the CONFIG RESETSTAT command.
 */
function reset() {

  // keyspace hits
  this.hits = 0;

  // keyspace misses
  this.misses = 0;

  // number of expired keys
  this.expired = 0;

  // total connections received
  this.connections = 0;

  // number of commands processed
  this.commands = 0;

  // number of rejected connections
  this.rejected = 0;
}

Stats.prototype.clear = clear;
Stats.prototype.reset = reset;
Stats.prototype.addStateListeners = addStateListeners;
Stats.prototype.removeStateListeners = removeStateListeners;

module.exports = Stats;
